@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001, 2002 Philip Blundell <philb@gnu.org>
+ * Copyright (C) 2001, 2002, 2003 Philip Blundell <philb@gnu.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -8,19 +8,17 @@
  */
 
 /*
- * $Id: main.c,v 1.42 2003/03/15 20:41:14 pb Exp $
+ * $Id: main.c,v 1.50 2003/08/19 09:50:28 florian Exp $
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
-
 #include <gtk/gtk.h>
 #include <libintl.h>
 #include <stdlib.h>
 #include <string.h>
-#include <libdm.h>
+#include <libdisplaymigration/displaymigration.h>
+#include <unistd.h>
+#include <locale.h>
 
 #include <gpe/init.h>
 #include <gpe/pixmaps.h>
@@ -38,26 +36,102 @@
 #include "proto.h"
 #include "callbacks.h"
 
-#define MY_PIXMAPS_DIR PREFIX "/share/gpe-contacts/pixmaps"
+#define MY_PIXMAPS_DIR PREFIX "/share/pixmaps/"
+#define TAB_CONFIG_LOCAL ".contacts-tab"
+#define TAB_CONFIG_GLOBAL "/etc/gpe/contacts-tab.conf"
 
 static GtkWidget *categories_smenu;
 gchar *active_chars;
-GtkWidget *mainw;		// how ugly making this global
+GtkWidget *mainw;		
 gboolean panel_config_has_changed = FALSE;
 GtkListStore *list_store;
 GtkWidget *list_view;
+static GtkWidget *bExport;
 
 struct gpe_icon my_icons[] = {
   {"edit"},
   {"delete"},
   {"cancel"},
-  {"frame", MY_PIXMAPS_DIR "/frame.png"},
-  {"notebook", MY_PIXMAPS_DIR "/notebook.png"},
-  {"entry", MY_PIXMAPS_DIR "/entry.png"},
-  {"icon", PREFIX "/share/pixmaps/gpe-contacts.png" },
+  {"frame", MY_PIXMAPS_DIR "frame.png"},
+  {"notebook", MY_PIXMAPS_DIR "notebook.png"},
+  {"entry", MY_PIXMAPS_DIR "entry.png"},
+  {"export", MY_PIXMAPS_DIR "export.png"},
+  {"icon", MY_PIXMAPS_DIR "gpe-contacts.png" },
   {NULL, NULL}
 };
 
+
+/* type that holds tab configuration data */
+typedef struct 
+{
+	char *label;
+	char *chars;
+}t_tabdef;
+
+t_tabdef *tabdefs = NULL;
+int num_tabs = 0;
+
+/* loads tab config from file */
+/* doesn't free existing structs! */
+static int 
+load_tab_config ()
+{
+  FILE *fnew;
+  char *label, *chars;
+  char *localfile;
+	
+  label = malloc(33);
+  chars = malloc(33);
+  num_tabs = 0;
+
+  localfile = g_strdup_printf ("%s/%s", g_get_home_dir (), TAB_CONFIG_LOCAL);
+  if (!access (localfile, R_OK))
+    fnew = fopen (localfile, "r");
+  else if (!access (TAB_CONFIG_GLOBAL, R_OK))
+    fnew = fopen (TAB_CONFIG_GLOBAL, "r");
+  else
+    fnew = NULL;
+
+  /* scans config file */  
+  /* TAB=<label> <charlist> */
+  if (fnew)
+    {
+      while (!feof (fnew) && (2 == fscanf (fnew, "TAB=%32s %32s\n", label, chars)))
+	{
+	  num_tabs++;
+	  tabdefs = realloc(tabdefs,num_tabs*sizeof(t_tabdef));
+	  tabdefs[num_tabs-1].label = g_strdup (label);
+	  tabdefs[num_tabs-1].chars = g_strdup (chars);
+	}
+      fclose (fnew);
+    }
+  else  // defaults
+    {
+      tabdefs = g_malloc (7*sizeof (t_tabdef));
+      tabdefs[0].label = g_strdup ("abcd");
+      tabdefs[0].chars = tabdefs[0].label;
+      tabdefs[1].label = g_strdup ("efgh");
+      tabdefs[1].chars = tabdefs[1].label;
+      tabdefs[2].label = g_strdup ("ijkl");
+      tabdefs[2].chars = tabdefs[2].label;
+      tabdefs[3].label = g_strdup ("mnop");
+      tabdefs[3].chars = tabdefs[3].label;
+      tabdefs[4].label = g_strdup ("qrstu");
+      tabdefs[4].chars = tabdefs[4].label;
+      tabdefs[5].label = g_strdup ("vwxyz");
+      tabdefs[5].chars = tabdefs[5].label;
+      tabdefs[6].label = g_strdup ("other");
+      tabdefs[6].chars = g_strdup ("!abcdefghijklmnopqrstuvwxyz");
+      num_tabs = 7;
+    }
+
+  free (label);
+  free (chars);
+  free (localfile);
+  active_chars = tabdefs[0].chars;
+  return num_tabs;
+}
+
 void
 load_panel_config ()
 {
@@ -71,7 +145,6 @@ load_panel_config ()
       table = lookup_widget (mainw, "tabDetail");
       container = lookup_widget (mainw, "pDetail");
       gtk_container_remove (GTK_CONTAINER (container), table);
-      //gtk_widget_unref(table);
       gtk_widget_destroy (table);
       table = gtk_table_new (1, 2, FALSE);
       gtk_widget_set_name (table, "tabDetail");
@@ -387,6 +460,37 @@ edit_person (struct person *p)
   gtk_widget_grab_focus (entry);
 }
 
+static void
+export_contact (GtkWidget * widget, gpointer d)
+{
+  GtkTreeSelection *sel = gtk_tree_view_get_selection (GTK_TREE_VIEW (list_view));
+  GtkTreeIter iter;
+  GtkTreeModel *model;
+  char *cmd;
+  GtkWidget *dlg;
+	
+  if (gtk_tree_selection_get_selected (sel, &model, &iter))
+    {
+      guint uid = -1;
+      gtk_tree_model_get (GTK_TREE_MODEL (model), &iter, 1, &uid, -1);
+	  if (uid >= 0)
+      {
+         cmd = g_strdup_printf("/usr/bin/vcard-export %i > /tmp/vcard.vcf",uid);
+         if (system(cmd))
+           dlg = gtk_message_dialog_new (GTK_WINDOW(mainw),
+              GTK_DIALOG_DESTROY_WITH_PARENT,GTK_MESSAGE_ERROR,
+              GTK_BUTTONS_CLOSE, _("Export of VCARD failed."));
+         else
+           dlg = gtk_message_dialog_new (GTK_WINDOW(mainw),
+              GTK_DIALOG_DESTROY_WITH_PARENT,GTK_MESSAGE_INFO, 
+              GTK_BUTTONS_CLOSE, _("VCARD exported to\n/tmp/vcard.vcf."));
+        gtk_dialog_run (GTK_DIALOG (dlg));
+        gtk_widget_destroy (dlg);
+        free(cmd);
+      }		  
+    }
+}
+
 static void
 new_contact (GtkWidget * widget, gpointer d)
 {
@@ -559,7 +663,7 @@ configure (GtkWidget * widget, gpointer d)
   gtk_window_set_title (GTK_WINDOW (window), _("Contacts: Configuration"));
   gpe_set_window_icon (window, "icon");
 
-  libdm_mark_window (window);
+  displaymigration_mark_window (window);
 
   gtk_widget_show_all (window);
 }
@@ -629,6 +733,12 @@ selection_made (GtkTreeSelection *sel)
       show_details (p);
 
       discard_person (p);
+		
+      gtk_widget_set_sensitive(bExport,TRUE);	
+    }
+  else
+    {
+       gtk_widget_set_sensitive(bExport,FALSE);
     }
 }
 
@@ -658,31 +768,7 @@ main_view_switch_page (GtkNotebook * notebook,
 		       GtkNotebookPage * page,
 		       gint page_num, gpointer user_data)
 {
-  switch (page_num)
-    {
-    case 0:
-      sprintf (active_chars, "ABC");
-      break;	
-    case 1:
-      sprintf (active_chars, "DEF");
-      break;	
-    case 2: 
-      sprintf (active_chars, "GHIJ");
-      break;	
-    case 3:
-      sprintf (active_chars, "KLMN");
-      break;	
-    case 4:
-      sprintf (active_chars, "OPQR");
-      break;	
-    case 5:
-      sprintf (active_chars, "STUV");
-      break;	
-    case 6:
-      sprintf (active_chars, "WXYZ");
-      break;	
-    }
-
+  active_chars = tabdefs[page_num].chars;
   update_display ();
 }
 
@@ -744,6 +830,8 @@ do_search (GObject *obj, GtkWidget *entry)
       g_slist_free (l);
     }
 
+  all_entries = g_slist_sort (all_entries, sort_entries);
+
   gtk_list_store_clear (list_store);
 
   for (iter = all_entries; iter; iter = iter->next)
@@ -772,12 +860,6 @@ create_main (void)
   GtkWidget *nbList;
   GtkWidget *empty_notebook_page;
   GtkWidget *label46a;
-  GtkWidget *label47;
-  GtkWidget *label48;
-  GtkWidget *label49;
-  GtkWidget *label50;
-  GtkWidget *label51;
-  GtkWidget *label52;
   GtkWidget *hbox3;
   GtkWidget *label83, *label84;
   GtkWidget *entry1;
@@ -789,6 +871,7 @@ create_main (void)
   GtkTreeSelection *tree_sel;
   GtkWidget *scrolled_window;
   GtkWidget *go_button;
+  int i;
 
   main_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_window_set_title (GTK_WINDOW (main_window), _("Contacts"));
@@ -819,66 +902,41 @@ create_main (void)
 			    _("Delete contact"), _("Tap here to delete the selected contact."),
 			    G_CALLBACK (delete_contact), NULL, -1);
 
+  pw = gtk_image_new_from_pixbuf (gpe_find_icon_scaled ("export", 
+							gtk_toolbar_get_icon_size (GTK_TOOLBAR (toolbar))));
+  bExport = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar), _("Export"),
+			    _("Export"), _("Tap here to export this contact to VCARD."),
+			    pw, G_CALLBACK (export_contact), NULL);
+				
   gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_PROPERTIES,
 			    _("Properties"), _("Tap here to configure the program."),
 			    G_CALLBACK (configure), NULL, -1);
+				
+  gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
+
+  gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar), GTK_STOCK_QUIT,
+			    _("Close"), _("Tap here to exit gpe-contacts."),
+			    G_CALLBACK (gtk_main_quit), NULL, -1);
+
+  gtk_widget_set_sensitive(bExport,FALSE);
 
   nbList = gtk_notebook_new ();
   gtk_box_pack_start (GTK_BOX (vbox1), nbList, FALSE, FALSE, 0);
   gtk_notebook_set_show_border (GTK_NOTEBOOK (nbList), FALSE);
   gtk_notebook_set_tab_hborder (GTK_NOTEBOOK (nbList), 1);
   gtk_notebook_set_tab_vborder (GTK_NOTEBOOK (nbList), 0);
+  gtk_notebook_set_scrollable(GTK_NOTEBOOK (nbList), TRUE);
 
-  empty_notebook_page = gtk_vbox_new (FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (nbList), empty_notebook_page);
-
-  label46a = gtk_label_new (_("ABC"));
-  gtk_widget_show (label46a);
-  gtk_notebook_set_tab_label (GTK_NOTEBOOK (nbList), 
-			      gtk_notebook_get_nth_page (GTK_NOTEBOOK (nbList), 0), label46a);
-
-  empty_notebook_page = gtk_vbox_new (FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (nbList), empty_notebook_page);
-
-  label47 = gtk_label_new (_("DEF"));
-  gtk_notebook_set_tab_label (GTK_NOTEBOOK (nbList), 
-			      gtk_notebook_get_nth_page (GTK_NOTEBOOK (nbList), 1), label47);
-
-  empty_notebook_page = gtk_vbox_new (FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (nbList), empty_notebook_page);
-
-  label48 = gtk_label_new (_("GHIJ"));
-  gtk_notebook_set_tab_label (GTK_NOTEBOOK (nbList), 
-			      gtk_notebook_get_nth_page (GTK_NOTEBOOK (nbList), 2), label48);
-
-  empty_notebook_page = gtk_vbox_new (FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (nbList), empty_notebook_page);
-
-  label49 = gtk_label_new (_("KLMN"));
-  gtk_notebook_set_tab_label (GTK_NOTEBOOK (nbList), 
-			      gtk_notebook_get_nth_page (GTK_NOTEBOOK (nbList), 3), label49);
-
-  empty_notebook_page = gtk_vbox_new (FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (nbList), empty_notebook_page);
-
-  label50 = gtk_label_new (_("OPQR"));
-  gtk_notebook_set_tab_label (GTK_NOTEBOOK (nbList), 
-			      gtk_notebook_get_nth_page (GTK_NOTEBOOK (nbList), 4), label50);
-
-  empty_notebook_page = gtk_vbox_new (FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (nbList), empty_notebook_page);
-
-  label51 = gtk_label_new (_("STUV"));
-  gtk_notebook_set_tab_label (GTK_NOTEBOOK (nbList), 
-			      gtk_notebook_get_nth_page (GTK_NOTEBOOK (nbList), 5), label51);
-
-  empty_notebook_page = gtk_vbox_new (FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (nbList), empty_notebook_page);
-
-  label52 = gtk_label_new (_("WXYZ"));
-  gtk_notebook_set_tab_label (GTK_NOTEBOOK (nbList), 
-			      gtk_notebook_get_nth_page (GTK_NOTEBOOK (nbList), 6), label52);
+  load_tab_config();
 
+  for (i=0;i<num_tabs;i++)
+  {
+     empty_notebook_page = gtk_vbox_new (FALSE, 0);
+     label46a = gtk_label_new(tabdefs[i].label);
+     gtk_widget_show (label46a);
+     gtk_notebook_append_page(GTK_NOTEBOOK(nbList),empty_notebook_page,label46a);	  
+  }
+  
   scrolled_window = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window), 
 				  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
@@ -929,8 +987,8 @@ create_main (void)
 
   g_signal_connect (G_OBJECT (tree_sel), "changed",
 		    G_CALLBACK (selection_made), NULL);
-
-  libdm_mark_window (main_window);
+ 
+  displaymigration_mark_window (main_window);
 
   return main_window;
 }
@@ -939,13 +997,13 @@ int
 main (int argc, char *argv[])
 {
 #ifdef ENABLE_NLS
+  setlocale (LC_ALL, "");
+
   bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
+  bind_textdomain_codeset (PACKAGE, "UTF-8");
   textdomain (PACKAGE);
 #endif
 	
-  active_chars = malloc (5 * sizeof (gchar));
-  sprintf (active_chars, "ABCD");
-
   if (gpe_application_init (&argc, &argv) == FALSE)
     exit (1);
 
@@ -955,7 +1013,7 @@ main (int argc, char *argv[])
   if (db_open ())
     exit (1);
 
-  libdm_init ();
+  displaymigration_init ();
 
   load_well_known_tags ();
 
